'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getAllWithKeys = getAllWithKeys;
exports.get = get;
exports.fromUrl = fromUrl;
exports.getFolders = getFolders;
exports.getFiles = getFiles;
exports.getFilesByType = getFilesByType;
exports.read = read;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _ = require('../../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getAllWithKeys(withKeys) {
  var files = _.cmsData.file.getFiles(_path2.default.join(_.config.root, _.config.data.url), true, 99, /\.json/);
  var filesArr = [];

  files.forEach(function (file) {
    var cleanFile = file;
    var json = _.cmsData.file.get(file.path);

    if (json.abe_meta.latest.date != null) {
      file.date = json.abe_meta.latest.date;
      file.cleanDate = (0, _moment2.default)(json.abe_meta.latest.date).format('YYYY/MM/DD HH:MM:ss');
    }

    if (json.abe_meta != null) {
      var date = null;
      if (json.abe_meta.latest.date !== null) {
        date = json.abe_meta.latest.date;
      } else if (json.abe_meta.date !== null) {
        date = json.abe_meta.date;
      }
      cleanFile.abe_meta = {
        date: date,
        type: json.abe_meta.type != null ? json.abe_meta.type : null,
        link: json.abe_meta.link != null ? json.abe_meta.link : null,
        template: json.abe_meta.template != null ? json.abe_meta.template : null,
        status: json.abe_meta.status != null ? json.abe_meta.status : null,
        cleanName: json.abe_meta.cleanName != null ? json.abe_meta.cleanName : null,
        cleanFilename: json.abe_meta.cleanFilename != null ? json.abe_meta.cleanFilename : null
      };
    }
    Array.prototype.forEach.call(withKeys, function (key) {
      var keyFirst = key.split('.')[0];
      cleanFile[keyFirst] = json[keyFirst];
    });
    filesArr.push(cleanFile);
  });

  var merged = _.cmsData.revision.getFilesMerged(filesArr);

  _.abeExtend.hooks.instance.trigger('afterGetAllFiles', merged);
  return merged;
}

function get(pathJson) {
  var json = {};
  pathJson = _.abeExtend.hooks.instance.trigger('beforeGetJson', pathJson);

  try {
    var stat = _fsExtra2.default.statSync(pathJson);
    if (stat) {
      json = _fsExtra2.default.readJsonSync(pathJson);
    }
  } catch (e) {}

  json = _.abeExtend.hooks.instance.trigger('afterGetJson', json);
  return json;
}

function fromUrl(url) {
  var res = {
    root: '',
    draft: {
      dir: '',
      file: '',
      path: ''
    },
    publish: {
      dir: '',
      file: '',
      link: '',
      path: '',
      json: ''
    },
    json: {
      path: '',
      file: ''
    }
  };

  if (url != null) {

    var dir = _path2.default.dirname(url).replace(_.config.root, '');
    var filename = _path2.default.basename(url);
    var link = url.replace(_.config.root, '');
    link = link.replace(/^\//, '').split('/');
    link.shift();
    link = _.cmsData.fileAttr.delete('/' + link.join('/').replace(/\/$/, ''));

    var draft = _.config.draft.url;
    var publish = _.config.publish.url;
    var data = _.config.data.url;

    res.root = _.config.root;

    // set dir path draft/json
    res.draft.dir = _.coreUtils.file.changePath(dir, draft);
    res.json.dir = _.coreUtils.file.changePath(dir, data);
    res.publish.dir = _.coreUtils.file.changePath(dir, publish);
    res.publish.json = res.json.dir;

    // set filename draft/json
    res.draft.file = filename;
    res.publish.file = _.cmsData.fileAttr.delete(filename);
    res.publish.link = link;
    res.json.file = filename.replace('.' + _.config.files.templates.extension, '.json');
    res.publish.json = _path2.default.join(res.json.dir, _.cmsData.fileAttr.delete(res.json.file));

    // set filename draft/json
    res.draft.path = _path2.default.join(res.draft.dir, res.draft.file);
    res.publish.path = _path2.default.join(res.publish.dir, res.publish.file);
    res.json.path = _path2.default.join(res.json.dir, res.json.file);
  }
  return res;
}

function getFolders(folder) {
  var flatten = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];
  var level = arguments[2];

  var arr = [];
  arr = _.cmsData.file.read(folder.replace(/\/$/, ''), folder.replace(/\/$/, ''), 'folders', flatten, /(.*?)/, level);

  return arr;
}

function getFiles(folder) {
  var flatten = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];
  var level = arguments[2];
  var extensions = arguments.length <= 3 || arguments[3] === undefined ? /(.*?)/ : arguments[3];
  var inversePattern = arguments.length <= 4 || arguments[4] === undefined ? false : arguments[4];


  var arr = [];
  arr = _.cmsData.file.read(folder.replace(/\/$/, ''), folder.replace(/\/$/, ''), 'files', flatten, extensions, level, 0, inversePattern);

  return arr;
}

function getFilesByType(pathFile) {
  var type = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  try {
    var directory = _fsExtra2.default.lstatSync(pathFile);
    if (!directory.isDirectory()) {
      _mkdirp2.default.sync(pathFile);
    }
  } catch (e) {
    _mkdirp2.default.sync(pathFile);
  }
  var files = _.cmsData.file.getFiles(pathFile, true, 20, new RegExp('.' + _.config.files.templates.extension));

  var result = [];

  Array.prototype.forEach.call(files, function (file) {
    var val = _.cmsData.fileAttr.get(file.path).s;
    if (type === null || val === type) result.push(file);
  });
  return result;
}

function read(base, dirName, type, flatten) {
  var extensions = arguments.length <= 4 || arguments[4] === undefined ? /(.*?)/ : arguments[4];
  var max = arguments.length <= 5 || arguments[5] === undefined ? 99 : arguments[5];
  var current = arguments.length <= 6 || arguments[6] === undefined ? 0 : arguments[6];
  var inversePattern = arguments.length <= 7 || arguments[7] === undefined ? false : arguments[7];

  var arr = [];
  var level = _fsExtra2.default.readdirSync(dirName);
  var fileCurrentLevel = [];
  var assets = _.config.files.templates.assets;

  for (var i = 0; i < level.length; i++) {
    var pathLevel = dirName + '/' + level[i];
    var isFolder = true;
    try {
      var directory = _fsExtra2.default.lstatSync(pathLevel);
      if (!directory.isDirectory()) {
        isFolder = false;
      }
    } catch (e) {
      isFolder = false;
    }
    var match = isFolder ? true : inversePattern ? !extensions.test(level[i]) : extensions.test(level[i]);
    if ((type === 'files' || type === null) && match) {

      if (level[i].indexOf('.') > -1) {
        var extension = /(\.[\s\S]*)/.exec(level[i])[0];
        var cleanName = _.cmsData.fileAttr.delete(level[i]);
        var cleanNameNoExt = cleanName.replace(/\..+$/, '');
        var fileData = _.cmsData.fileAttr.get(level[i]);

        var date;
        if (fileData.d) {
          date = fileData.d;
        } else {
          var stat = _fsExtra2.default.statSync(pathLevel);
          date = stat.mtime;
        }
        var cleanFilePath = _.cmsData.fileAttr.delete(pathLevel).replace(_.config.root, '').replace(/^\/?.+?\//, '');

        var fileDate = (0, _moment2.default)(date);
        var duration = _moment2.default.duration((0, _moment2.default)(fileDate).diff(new Date())).humanize(true);

        var filePath = pathLevel.replace(_.config.root, '');
        filePath = filePath.split('/');
        filePath.shift();
        filePath = filePath.join('/');
        var item = {
          'name': level[i],
          'path': pathLevel,
          'cleanPathName': _.cmsData.fileAttr.delete(pathLevel),
          'cleanPath': pathLevel.replace(base + '/', ''),
          date: date,
          cleanDate: fileDate.format('YYYY/MM/DD HH:MM:ss'),
          duration: duration,
          cleanName: cleanName,
          cleanNameNoExt: cleanNameNoExt,
          cleanFilePath: cleanFilePath,
          filePath: filePath,
          'type': 'file',
          'fileType': extension
        };

        if (!flatten) item['folders'] = [];
        arr.push(item);
        // push current file name into array to check if siblings folder are assets folder
        fileCurrentLevel.push(level[i].replace(/\..+$/, '') + assets);
      }
    }
    if (!fileCurrentLevel.includes(level[i]) && match) {
      if (isFolder) {
        if (!flatten) {
          var index = arr.push({ 'name': level[i], 'path': pathLevel, 'cleanPath': pathLevel.replace(base + '/', ''), 'folders': [], 'type': 'folder' }) - 1;
          if (current < max) {
            arr[index].folders = _.cmsData.file.read(base, pathLevel, type, flatten, extensions, max, current + 1, inversePattern);
          }
        } else {
          if (type === 'folders' || type === null) {
            arr.push({ 'name': level[i], 'path': pathLevel, 'cleanPath': pathLevel.replace(base + '/', ''), 'type': 'folder' });
          }
          if (current < max) {
            Array.prototype.forEach.call(_.cmsData.file.read(base, pathLevel, type, flatten, extensions, max, current + 1, inversePattern), function (files) {
              arr.push(files);
            });
          }
        }
      }
    }
  }

  return arr;
}