'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getFilesRevision = getFilesRevision;
exports.getVersions = getVersions;
exports.getDocumentRevision = getDocumentRevision;
exports.getStatusAndDateToFileName = getStatusAndDateToFileName;
exports.removeStatusAndDateFromFileName = removeStatusAndDateFromFileName;
exports.filePathInfos = filePathInfos;
exports.getFilesMerged = getFilesMerged;
exports.deleteOlderRevisionByType = deleteOlderRevisionByType;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ = require('../../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getFilesRevision(urls, fileName) {
  var res = [];
  var number = 1;
  var tplUrl = _.cmsData.file.fromUrl(fileName);
  fileName = fileName.split('/');
  fileName = fileName[fileName.length - 1];
  var publishDate = new Date();
  var json = null;

  if (_.coreUtils.file.exist(tplUrl.publish.json)) {
    json = _.cmsData.file.get(tplUrl.publish.json);
    if (json != null && json[_.config.meta.name] != null) {
      publishDate = new Date(json[_.config.meta.name].latest.date);
    }
  }

  var publishVersion = false;
  urls.forEach(function (urlObj) {
    var fileData = _.cmsData.fileAttr.get(urlObj.cleanPath);
    if (fileData.s === 'd' && _.cmsData.fileAttr.delete(urlObj.cleanPath) == _.cmsData.fileAttr.delete(fileName)) {
      var currentDate = new Date(urlObj.date);
      if (currentDate.getTime() > publishDate.getTime()) {
        if (!publishVersion && res[res.length - 1] != null) {
          res[res.length - 1].publishedDate = 'same';
        }
        publishVersion = true;
        urlObj.publishedDate = 'after';
      } else if (currentDate.getTime() === publishDate.getTime()) {
        urlObj.publishedDate = 'same';
        publishVersion = true;
      } else {
        urlObj.publishedDate = 'before';
      }
      urlObj.version = number;
      number = number + 1;

      var tplUrlObj = _.cmsData.file.fromUrl(urlObj.path);
      if (_.coreUtils.file.exist(tplUrlObj.publish.json)) {
        var jsonObj = _.cmsData.file.get(tplUrlObj.publish.json);
        urlObj[_.config.meta.name] = jsonObj[_.config.meta.name];
      }
      res.push(urlObj);
    }
  });
  return res;
}

/**
 * Create and array of doc file path containing the versions of this doc
 * @param  {String} path of a doc
 * @return {Array} the versions of the doc
 */
function getVersions(docPath) {
  var result = [];
  var files = _.Manager.instance.getList();
  var dataFile = _path2.default.join(_.config.data.url, docPath.replace('.' + _.config.files.templates.extension, '.json'));
  Array.prototype.forEach.call(files, function (file) {
    if (file.path.indexOf(dataFile) > -1) {
      result = file.revisions;
    }
  });
  return result;
}

/**
 * Return the revision from document html file path
 * if the docPath contains abe revision [status]-[date] will try to return this revision
 * else it will return the latest revision
 * or null
 * 
 * @param  {String} html path
 * @return {Object} file revision | null
 */
function getDocumentRevision(docPath) {
  var result = null;
  var documentPath = docPath;
  var latest = true;

  if (_.cmsData.fileAttr.test(documentPath)) {
    latest = false;
    documentPath = _.cmsData.fileAttr.delete(documentPath);
  }
  var revisions = getVersions(documentPath);
  if (latest && revisions.length >= 0) {
    result = revisions[0];
  } else if (!latest) {
    Array.prototype.forEach.call(revisions, function (revision) {
      if (revision.html === docPath) {
        result = revision;
      }
    });
    if (result === null && revisions.length >= 0) {
      result = revisions[0];
    }
  }
  return result;
}

function getStatusAndDateToFileName(date) {
  var res = date.substring(0, 4) + '-' + date.substring(4, 6) + '-' + date.substring(6, 11) + ':' + date.substring(11, 13) + ':' + date.substring(13, 15) + '.' + date.substring(15, 19);
  return res;
}

function removeStatusAndDateFromFileName(date) {
  return date.replace(/[-:\.]/g, '');
}

function filePathInfos(pathFolder) {
  var pathArr = pathFolder.split('/');
  var name = pathArr[pathArr.length - 1];

  var rootArr = _.config.root.split('/');
  var website = rootArr[pathArr.length - 1];
  return {
    'name': name,
    'path': pathFolder,
    'website': website,
    'cleanPath': pathFolder.replace(_.config.root, '').replace(/\/$/, ''),
    'type': 'folder'
  };
}

function getFilesMerged(files) {
  var merged = {};
  var arMerged = [];

  Array.prototype.forEach.call(files, function (file) {
    var cleanFilePath = file.cleanFilePath;

    var fileStatusIsPublish = _.cmsData.fileAttr.get(file.cleanPath);
    if (fileStatusIsPublish.s != null && file.abe_meta.status === 'publish') {
      file.abe_meta.status = 'draft';
    }

    file.html = _path2.default.join('/', file.filePath.replace(/\.json/, '.' + _.config.files.templates.extension));
    if (file.abe_meta.status === 'publish') {
      file.htmlPath = _path2.default.join(_.config.root, _.config.publish.url, _path2.default.join('/', file.filePath.replace(/\.json/, '.' + _.config.files.templates.extension)));
    } else {
      file.htmlPath = _path2.default.join(_.config.root, _.config.draft.url, _path2.default.join('/', file.filePath.replace(/\.json/, '.' + _.config.files.templates.extension)));
    }

    if (merged[cleanFilePath] == null) {
      merged[cleanFilePath] = {
        name: _.cmsData.fileAttr.delete(file.name),
        path: _.cmsData.fileAttr.delete(file.path),
        html: _.cmsData.fileAttr.delete(_path2.default.join('/', file.filePath.replace(/\.json/, '.' + _.config.files.templates.extension))),
        htmlPath: _path2.default.join(_.config.root, _.config.publish.url, _path2.default.join('/', _.cmsData.fileAttr.delete(file.filePath.replace(/\.json/, '.' + _.config.files.templates.extension)))),
        cleanPathName: file.cleanPathName,
        cleanPath: file.cleanPath,
        cleanName: file.cleanName,
        cleanNameNoExt: file.cleanNameNoExt,
        cleanFilePath: file.cleanFilePath,
        filePath: _.cmsData.fileAttr.delete(file.filePath),
        revisions: []
      };
    }

    merged[cleanFilePath].revisions.push(JSON.parse(JSON.stringify(file)));
  });

  // return merged
  Array.prototype.forEach.call(Object.keys(merged), function (key) {
    var revisions = merged[key].revisions;

    revisions.sort(_.coreUtils.sort.predicatBy('date', -1));
    if (revisions[0] != null) {
      merged[key].date = revisions[0].date;
    }
    Array.prototype.forEach.call(revisions, function (revision) {
      var status = revision.abe_meta.status;
      if (typeof merged[key][status] === 'undefined' || merged[key][status] === null) {
        if (status === 'publish') {
          merged[key][status] = revision;
        } else {
          merged[key][status] = {};
        }
        merged[key][status].path = revision.path;
        merged[key][status].html = revision.html;
        merged[key][status].htmlPath = revision.htmlPath;
        merged[key][status].date = new Date(revision.date);
        merged[key][status].cleanDate = revision.cleanDate;
        merged[key][status].link = revision.abe_meta.link;
      }
    });

    merged[key].revisions = revisions;

    merged[key].date = revisions[0].date;
    merged[key].cleanDate = revisions[0].cleanDate;
    merged[key].duration = revisions[0].duration;
    merged[key].abe_meta = revisions[0].abe_meta;

    arMerged.push(merged[key]);
  });

  return arMerged;
}

function deleteOlderRevisionByType(fileName, type) {
  var folder = fileName.split('/');
  var file = folder.pop();
  var extension = file.replace(/.*?\./, '');
  folder = folder.join('/');
  try {
    var stat = _fsExtra2.default.statSync(folder);
    if (stat) {
      var files = _.cmsData.file.getFiles(folder, true, 1, new RegExp('\\.' + extension));
      files.forEach(function (fileItem) {
        var fname = _.cmsData.fileAttr.delete(fileItem.cleanPath);
        var ftype = _.cmsData.fileAttr.get(fileItem.cleanPath).s;
        if (fname === file && ftype === type) {
          var fileDraft = fileItem.path.replace(/-abe-./, '-abe-d');
          _.cmsOperations.remove.removeFile(fileItem.path, _.coreUtils.file.changePath(fileItem.path, _.config.data.url).replace(new RegExp('\\.' + extension), '.json'));
          _.cmsOperations.remove.removeFile(fileDraft, _.coreUtils.file.changePath(fileDraft, _.config.data.url).replace(new RegExp('\\.' + extension), '.json'));
        }
      });
    }
  } catch (e) {}
}