'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.checkRequired = checkRequired;
exports.save = save;
exports.saveJsonAndHtml = saveJsonAndHtml;
exports.saveJson = saveJson;
exports.saveHtml = saveHtml;
exports.dateIso = dateIso;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _xss = require('xss');

var _xss2 = _interopRequireDefault(_xss);

var _es6Promise = require('es6-promise');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ = require('../../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function checkRequired(text, json) {
  var regAbe = /{{abe[\S\s].*?key=['|"]([\S\s].*?['|"| ]}})/g;
  var matches = text.match(regAbe);
  var requiredValue = 0;
  var complete = 0;
  if (typeof matches !== 'undefined' && matches !== null) {
    Array.prototype.forEach.call(matches, function (match) {
      if (typeof match !== 'undefined' && match !== null) {

        var keyAttr = _.cmsData.regex.getAttr(match, 'key');
        var requiredAttr = _.cmsData.regex.getAttr(match, 'required');
        if (requiredAttr === 'true') {
          requiredValue++;

          var minAttr = _.cmsData.regex.getAttr(match, 'min-length');
          minAttr = minAttr !== '' ? minAttr : 0;

          if (typeof json[keyAttr] !== 'undefined' && json[keyAttr] !== null && json[keyAttr] !== '') {
            if (minAttr > 0) {
              if (json[keyAttr].length >= minAttr) {
                complete++;
              }
            } else {
              complete++;
            }
          }
        }
      }
    });
  }

  return Math.round(requiredValue > 0 ? complete * 100 / requiredValue : 100);
}

function save(url, tplPath) {
  var json = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];
  var text = arguments.length <= 3 || arguments[3] === undefined ? '' : arguments[3];
  var type = arguments.length <= 4 || arguments[4] === undefined ? '' : arguments[4];
  var previousSave = arguments.length <= 5 || arguments[5] === undefined ? null : arguments[5];
  var realType = arguments.length <= 6 || arguments[6] === undefined ? 'draft' : arguments[6];
  var publishAll = arguments.length <= 7 || arguments[7] === undefined ? false : arguments[7];

  url = _.coreUtils.slug.clean(url);

  var p = new _es6Promise.Promise(function (resolve) {
    var isRejectedDoc = false;
    if (type === 'reject') {
      isRejectedDoc = true;
      url = _.abeExtend.hooks.instance.trigger('beforeReject', url);
      type = 'draft';
      realType = 'draft';
      url = _.abeExtend.hooks.instance.trigger('afterReject', url);
    }
    var tplUrl = _.cmsData.file.fromUrl(url);
    type = type || 'draft';
    var pathIso = dateIso(tplUrl, type);
    if (typeof previousSave !== 'undefined' && previousSave !== null) {
      pathIso.jsonPath = _path2.default.join(_.config.root, previousSave.jsonPath.replace(_.config.root, '')).replace(/-abe-d/, '-abe-' + realType[0]);
      pathIso.htmlPath = _path2.default.join(_.config.root, previousSave.htmlPath.replace(_.config.root, '')).replace(/-abe-d/, '-abe-' + realType[0]);
    }

    if (tplPath.indexOf('.') > -1) {
      tplPath = tplPath.replace(/\..+$/, '');
    }
    var tpl = tplPath.replace(_.config.root, '');

    var fullTpl = _path2.default.join(_.config.root, _.config.templates.url, tpl) + '.' + _.config.files.templates.extension;

    if (typeof json === 'undefined' || json === null) {
      json = _.cmsData.file.get(tplUrl.json.path);
    }

    var ext = {
      template: tpl.replace(/^\/+/, ''),
      link: tplUrl.publish.link,
      complete: 0,
      type: type
    };
    var meta = _.config.meta.name;
    json[meta] = (0, _extend2.default)(json[meta], ext);
    var date = _.cmsData.fileAttr.get(pathIso.jsonPath).d;

    if (publishAll) {
      if (typeof json[meta].publish !== 'undefined' && json[meta].publish !== null) {
        date = json[meta].publish.date;
      }
    } else {
      if (typeof date === 'undefined' || date === null || date === '') {
        date = new Date();
      } else {
        date = new Date(date);
      }
    }

    _.cmsData.metas.add(tpl, json, type, {}, date, realType);

    if (typeof text === 'undefined' || text === null || text === '') {
      text = _.cmsTemplates.template.getTemplate(fullTpl);
    }

    _.cmsData.source.getDataList(_path2.default.dirname(tplUrl.publish.link), text, json).then(function () {

      json = _.abeExtend.hooks.instance.trigger('afterGetDataListOnSave', json);
      for (var prop in json) {
        if (_typeof(json[prop]) === 'object' && Array.isArray(json[prop]) && json[prop].length === 1) {
          var valuesAreEmpty = true;
          json[prop].forEach(function (element) {
            for (var p in element) {
              if (element[p] !== '') {
                valuesAreEmpty = false;
              }
            }
          });
          if (valuesAreEmpty) {
            delete json[prop];
          }
        }
      }

      var obj = {
        publishAll: publishAll,
        type: type,
        template: {
          path: fullTpl
        },
        html: {
          path: pathIso.htmlPath
        },
        json: {
          content: json,
          path: pathIso.jsonPath
        }
      };

      obj = _.abeExtend.hooks.instance.trigger('beforeSave', obj);

      obj.json.content[meta].complete = checkRequired(text, obj.json.content);

      var res = saveJsonAndHtml(tpl.replace(/^\/+/, ''), obj, text);
      if (isRejectedDoc) {
        res.reject = _.cmsData.fileAttr.delete(url).replace(_path2.default.join(_.config.root, _.config.draft.url), '');
      }

      _.abeExtend.hooks.instance.trigger('afterSave', obj);

      _.cmsTemplates.assets.copy();

      resolve(res);
    }).catch(function (e) {
      console.error('Save.js', e);
    });
  });

  return p;
}

function saveJsonAndHtml(templateId, obj, html) {
  var page = new _.Page(templateId, html, obj.json.content, true);

  if (obj.json.content.abe_meta.status === 'publish') {
    saveHtml(obj.html.path, page.html);
  }
  saveJson(obj.json.path, obj.json.content);

  return {
    json: obj.json.content,
    jsonPath: obj.json.path,
    html: page.html,
    htmlPath: obj.html.path
  };
}

function saveJson(url, json) {
  _mkdirp2.default.sync(_path2.default.dirname(url));

  if (typeof json.abe_source !== 'undefined' && json.abe_source !== null) {
    delete json.abe_source;
  }

  var eachRecursive = function eachRecursive(obj) {
    for (var k in obj) {
      if (_typeof(obj[k]) === 'object' && obj[k] !== null) {
        eachRecursive(obj[k]);
      } else if (typeof obj[k] !== 'undefined' && obj[k] !== null) {
        obj[k] = (0, _xss2.default)(obj[k].toString().replace(/&quot;/g, '"'), { 'whiteList': _.config.htmlWhiteList });
      }
    }
  };

  eachRecursive(json);

  _fsExtra2.default.writeJsonSync(url, json, {
    space: 2,
    encoding: 'utf-8'
  });
  return true;
}

function saveHtml(url, html) {
  _mkdirp2.default.sync(_path2.default.dirname(url));
  if (_.cmsData.fileAttr.test(url) && _.cmsData.fileAttr.get(url).s !== 'd') {
    _.cmsOperations.remove.olderRevisionByType(_.cmsData.fileAttr.delete(url), _.cmsData.fileAttr.get(url).s);
  }
  _fsExtra2.default.writeFileSync(url, html);
}

function dateIso(tplUrl) {
  var type = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  var newDateISO;
  var dateISO;
  var saveJsonFile = tplUrl.json.path;
  var saveFile = tplUrl['draft'].path;

  switch (type) {
    case 'draft':
      newDateISO = _.cmsData.revision.removeStatusAndDateFromFileName(new Date().toISOString());
      dateISO = 'd' + newDateISO;
      break;
    case 'publish':
      saveJsonFile = tplUrl.publish.json;
      saveFile = tplUrl.publish.path;
      break;
    default:
      newDateISO = _.cmsData.revision.removeStatusAndDateFromFileName(new Date().toISOString());
      dateISO = type[0] + newDateISO;
      break;
  }

  if (dateISO) {
    saveJsonFile = _.cmsData.fileAttr.add(saveJsonFile, dateISO);
    saveFile = _.cmsData.fileAttr.add(saveFile, dateISO);
  }

  return {
    jsonPath: saveJsonFile,
    htmlPath: saveFile
  };
}