'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.findTemplateAndPartialsInFolder = findTemplateAndPartialsInFolder;
exports.getTemplateAndPartials = getTemplateAndPartials;
exports.addOrder = addOrder;
exports.getAbeImport = getAbeImport;
exports.includePartials = includePartials;
exports.getTemplate = getTemplate;
exports.getVariablesInWhere = getVariablesInWhere;
exports.recurseWhereVariables = recurseWhereVariables;
exports.execRequestColumns = execRequestColumns;
exports.findRequestColumns = findRequestColumns;
exports.getSelectTemplateKeys = getSelectTemplateKeys;
exports.getStructureAndTemplates = getStructureAndTemplates;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _es6Promise = require('es6-promise');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ = require('../../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function findTemplateAndPartialsInFolder(currentPath) {
  var res = [];
  var files = _fsExtra2.default.readdirSync(currentPath);
  for (var i in files) {
    var currentFile = currentPath + '/' + files[i];
    var stats = _fsExtra2.default.statSync(currentFile);
    if (stats.isFile()) {
      if (currentFile.indexOf('.' + _.config.files.templates.extension) > -1) {
        res.push(currentFile);
      }
    } else if (stats.isDirectory()) {
      res = res.concat(findTemplateAndPartialsInFolder(currentFile));
    }
  }
  return res;
}

function getTemplateAndPartials(templatesPath) {
  var p = new _es6Promise.Promise(function (resolve) {
    var templatesList = findTemplateAndPartialsInFolder(templatesPath);
    resolve(templatesList);
  });

  return p;
}

function addOrder(text) {
  var regAbe = /{{abe[\S\s].*?key=['|"]([\S\s].*?['|"| ]}})/g;
  var matches = text.match(regAbe);
  var order = 0;

  if (typeof matches !== 'undefined' && matches !== null) {
    Array.prototype.forEach.call(matches, function (match) {
      if (typeof match !== 'undefined' && match !== null) {

        var orderAttr = _.cmsData.regex.getAttr(match, 'order');

        if (typeof orderAttr === 'undefined' || orderAttr === null || orderAttr === '') {
          var matchOrder = match.replace(/\}\}$/, ' order=\'' + order + '\'}}');
          text = text.replace(match, matchOrder);
        }
        order++;
      }
    });
  }
  return text;
}

function getAbeImport(text) {
  var partials = [];
  var listReg = /({{abe.*?type=[\'|\"]import.*?}})/g;
  var match;
  while (match = listReg.exec(text)) {
    partials.push(match[0]);
  }

  return partials;
}

function includePartials(text) {
  var abeImports = getAbeImport(text);

  Array.prototype.forEach.call(abeImports, function (abeImport) {
    var obj = _.cmsData.attributes.getAll(abeImport, {});

    var file = obj.file;
    var partial = '';
    file = _path2.default.join(_.config.root, _.config.partials, file);
    if (_.coreUtils.file.exist(file)) {
      partial = includePartials(_fsExtra2.default.readFileSync(file, 'utf8'));
    }
    text = text.replace(_.cmsData.regex.escapeTextToRegex(abeImport, 'g'), partial);
  });

  return text;
}

function translate(text) {
  var importReg = /({{abe.*type=[\'|\"]translate.*}})/g;

  var matches = text.match(importReg);

  if (typeof matches !== 'undefined' && matches !== null) {
    Array.prototype.forEach.call(matches, function (match) {
      var splitedMatches = match.split('{{abe ');

      Array.prototype.forEach.call(splitedMatches, function (splitedMatch) {
        var currentMatch = '{{abe ' + splitedMatch;
        if (/({{abe.*type=[\'|\"]translate.*}})/.test(currentMatch)) {
          var locale = _.cmsData.regex.getAttr(currentMatch, 'locale');
          var source = _.cmsData.regex.getAttr(currentMatch, 'source');

          if (locale.indexOf('{{') === -1) {
            locale = '\'' + locale + '\'';
          } else {
            locale = locale.replace(/\{\{(.*?)\}\}/, '$1');
          }

          if (source.indexOf('{{') === -1) {
            source = '\'' + source.replace(/'/g, '\\\'') + '\'';
          } else {
            source = source.replace(/\{\{(.*?)\}\}/, '$1');
          }

          // var replace = `{{{i18nAbe ${locale} ${source}}}}`
          var replace = currentMatch.replace('{{abe', '{{i18nAbe');
          replace = replace.replace(/locale=['|"].*?['|"]/, locale);
          replace = replace.replace(/source=['|"].*?['|"]/, source);
          replace = replace.replace(/{{i18nAbe.*?}}/, '{{{i18nAbe ' + locale + ' ' + source + '}}}');

          text = text.replace(_.cmsData.regex.escapeTextToRegex(currentMatch, 'g'), replace);
        }
      });
    });
  }

  return text;
}

function getTemplate(file) {
  var text = '';

  // HOOKS beforeGetTemplate
  file = _.abeExtend.hooks.instance.trigger('beforeGetTemplate', file);

  file = file.replace(_path2.default.join(_.config.root, _.config.templates.url), '');
  file = file.replace(_.config.root, '');
  if (file.indexOf('.') > -1) {
    file = file.replace(/\..+$/, '');
  }
  file = _path2.default.join(_.config.root, _.config.templates.url, file + '.' + _.config.files.templates.extension);
  if (_.coreUtils.file.exist(file)) {
    text = _fsExtra2.default.readFileSync(file, 'utf8');
    text = includePartials(text);
    text = translate(text);
    text = addOrder(text);
  } else {
    text = '[ ERROR ] template ' + (file + '.' + _.config.files.templates.extension) + ' doesn\'t exist anymore';
  }

  // HOOKS afterGetTemplate
  text = _.abeExtend.hooks.instance.trigger('afterGetTemplate', text);

  return text;
}

function getVariablesInWhere(where) {
  var ar = [];

  if (where.left.column.indexOf('{{') > -1) {
    ar.push(where.left.column.replace(/\{\{(.*?)\}\}/, '$1'));
  } else {
    ar.push(where.left.column);
  }

  if (where.right.value) {
    if (typeof where.right.value === 'string') {
      if (where.right.value && where.right.value.indexOf('{{') > -1) {
        ar.push(where.right.value.replace(/\{\{(.*?)\}\}/, '$1'));
      }
    } else {
      where.right.value.forEach(function (value) {
        if (value.column.indexOf('{{') > -1) {
          ar.push(value.column.replace(/\{\{(.*?)\}\}/, '$1'));
        }
      });
    }
  }

  if (where.right.column && where.right.column.indexOf('{{') > -1) {
    ar.push(where.right.column.replace(/\{\{(.*?)\}\}/, '$1'));
  }

  return ar;
}

/**
 * Get columns and where.left ids of a select statement
 *
 * select title, image from ../ where template=""
 *
 * return [title, image, template]
 * 
 * @param  {Array} templatesList ["article.html", "other.html"]
 * @return {Promise}
 */
function recurseWhereVariables(where) {
  var ar = [];
  var arLeft;
  var arRight;
  switch (where.operator) {
    case 'AND':
      arLeft = recurseWhereVariables(where.left);
      arRight = recurseWhereVariables(where.right);
      return arLeft.concat(arRight);
      break;
    case 'OR':
      arLeft = recurseWhereVariables(where.left);
      arRight = recurseWhereVariables(where.right);
      return arLeft.concat(arRight);
      break;
    default:
      ar = getVariablesInWhere(where);
      break;
  }

  return ar;
}

function execRequestColumns(tpl) {
  var ar = [];
  var matches = _.cmsData.regex.getTagAbeTypeRequest(tpl);
  Array.prototype.forEach.call(matches, function (match) {
    var obj = _.cmsData.attributes.getAll(match[0], {});
    var type = _.cmsData.sql.getSourceType(obj.sourceString);
    switch (type) {
      case 'request':
        var request = _.cmsData.sql.handleSqlRequest(obj.sourceString, {});
        if (typeof request.columns !== 'undefined' && request.columns !== null) {
          Array.prototype.forEach.call(request.columns, function (column) {
            ar.push(column);
          });
        }
        if (typeof request.where !== 'undefined' && request.where !== null) {
          ar = ar.concat(recurseWhereVariables(request.where));
        }
    }
  });

  return ar;
}

function findRequestColumns(templatesList) {
  var whereKeys = [];
  var p = new _es6Promise.Promise(function (resolve) {
    Array.prototype.forEach.call(templatesList, function (file) {
      var template = _fsExtra2.default.readFileSync(file, 'utf8');
      whereKeys = whereKeys.concat(execRequestColumns(template));
    });
    whereKeys = whereKeys.filter(function (item, pos) {
      return whereKeys.indexOf(item) == pos;
    });
    resolve(whereKeys);
  });

  return p;
}

function getSelectTemplateKeys(templatesPath) {
  var p = new _es6Promise.Promise(function (resolve, reject) {
    getTemplateAndPartials(templatesPath).then(function (templatesList) {
      findRequestColumns(templatesList).then(function (whereKeys) {
        resolve(whereKeys);
      }, function (e) {
        console.log('findRequestColumns reject');
        reject(e);
      }).catch(function (e) {
        console.error('getSelectTemplateKeys', e);
        reject();
      });
    }, function () {
      console.log('getTemplateAndPartials reject');
      reject();
    }).catch(function (e) {
      console.error('getSelectTemplateKeys', e);
      reject();
    });
  });

  return p;
}

function getStructureAndTemplates() {
  var site = _.cmsData.revision.filePathInfos(_.config.root);
  var result = { 'structure': [], 'templates': [] };

  var structure = _path2.default.join(site.path, _.config.structure.url);
  var templates = _path2.default.join(site.path, _.config.templates.url);
  try {
    var directoryStructure = _fsExtra2.default.lstatSync(structure);
    if (directoryStructure.isDirectory()) {
      result.structure = _.cmsData.file.getFolders(structure, false);
    }
  } catch (e) {}
  try {
    var directoryTemplate = _fsExtra2.default.lstatSync(templates);
    if (directoryTemplate.isDirectory()) {
      var resultTemplates = result.templates.concat(_.cmsData.file.getFiles(templates, true, 10, new RegExp('.' + _.config.files.templates.extension)));
      result.templates = resultTemplates.filter(function (resultTemplate) {
        return resultTemplate.path.indexOf(_.config.partials) < 0;
      });
    }
  } catch (e) {}

  return result;
}