'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _handlebars = require('handlebars');

var _handlebars2 = _interopRequireDefault(_handlebars);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _watch = require('watch');

var _watch2 = _interopRequireDefault(_watch);

var _ = require('../../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var singleton = Symbol();
var singletonEnforcer = Symbol();

var Manager = function () {
  function Manager(enforcer) {
    _classCallCheck(this, Manager);

    if (enforcer != singletonEnforcer) throw 'Cannot construct Json singleton';

    _handlebars2.default.templates = _handlebars2.default.templates || {};
    this.loadHbsTemplates();
  }

  _createClass(Manager, [{
    key: 'init',
    value: function init() {
      var _this = this;

      this._pathTemplate = _path2.default.join(_.config.root, _.config.templates.url);
      this._pathStructure = _path2.default.join(_.config.root, _.config.structure.url);
      this._watchersStart();

      this.updateStructureAndTemplates();
      var p = new Promise(function (resolve) {
        _this.getKeysFromSelect().then(function () {
          resolve();
        }, function (e) {
          console.log('Manager.init', e);
          resolve();
        }).catch(function (e) {
          console.log('Manager.init', e);
        });
      });

      return p;
    }
  }, {
    key: '_watchersStart',
    value: function _watchersStart() {
      var _this2 = this;

      this.events = {
        template: new _events2.default.EventEmitter(0),
        structure: new _events2.default.EventEmitter(0)
      };

      this._watchTemplateFolder = _watch2.default.createMonitor(this._pathTemplate, function (monitor) {
        monitor.on('created', function (f, stat) {
          _this2.getKeysFromSelect();
          _this2.updateStructureAndTemplates();
          _this2.events.template.emit('update');
        });
        monitor.on('changed', function (f, curr, prev) {
          _this2.getKeysFromSelect();
          _this2.updateStructureAndTemplates();
          _this2.events.template.emit('update');
        });
        monitor.on('removed', function (f, stat) {
          _this2.getKeysFromSelect();
          _this2.updateStructureAndTemplates();
          _this2.events.template.emit('update');
        });
      });

      this._watchStructure = _watch2.default.createMonitor(this._pathStructure, function (monitor) {
        monitor.on('created', function (f, stat) {
          _this2.updateStructureAndTemplates();
          _this2.events.structure.emit('update');
        });
        monitor.on('changed', function (f, curr, prev) {
          _this2.updateStructureAndTemplates();
          _this2.events.structure.emit('update');
        });
        monitor.on('removed', function (f, stat) {
          _this2.updateStructureAndTemplates();
          _this2.events.structure.emit('update');
        });
      });
    }
  }, {
    key: 'getKeysFromSelect',
    value: function getKeysFromSelect() {
      var _this3 = this;

      this._whereKeys = [];
      var p = new Promise(function (resolve) {
        _.cmsTemplates.template.getSelectTemplateKeys(_this3._pathTemplate).then(function (whereKeys) {
          _this3._whereKeys = whereKeys;
          _this3.updateList();
          resolve();
        }, function (e) {
          console.log('Manager.getKeysFromSelect', e);
        }).catch(function (e) {
          console.log('Manager.getKeysFromSelect', e);
        });
      });

      return p;
    }
  }, {
    key: 'getStructureAndTemplates',
    value: function getStructureAndTemplates() {

      return this._structureAndTemplates;
    }
  }, {
    key: 'updateStructureAndTemplates',
    value: function updateStructureAndTemplates() {
      this._structureAndTemplates = _.cmsTemplates.template.getStructureAndTemplates();
    }
  }, {
    key: 'getList',
    value: function getList() {

      return this._list;
    }
  }, {
    key: 'getListWithStatusOnFolder',
    value: function getListWithStatusOnFolder(status) {
      var folder = arguments.length <= 1 || arguments[1] === undefined ? '' : arguments[1];

      var list = [];
      folder = _path2.default.join(_.config.root, _.config.data.url, folder);
      Array.prototype.forEach.call(this._list, function (file) {
        if (typeof file[status] !== 'undefined' && file[status] !== null && file.path.indexOf(folder) > -1) {
          list.push(file);
        }
      });

      return list;
    }
  }, {
    key: 'setList',
    value: function setList(list) {
      this._list = list;

      return this;
    }
  }, {
    key: 'updateList',
    value: function updateList() {
      this._list = _.cmsData.file.getAllWithKeys(this._whereKeys);
      this._list.sort(_.coreUtils.sort.predicatBy('date', -1));
      console.log('Manager updated');

      return this;
    }
  }, {
    key: 'addHbsTemplate',
    value: function addHbsTemplate(templateId) {
      var pathTemplate = _path2.default.join(_.config.root, _.config.templates.url, 'hbs', templateId) + '.hbs';
      var tmpl = eval('(function(){return ' + _fsExtra2.default.readFileSync(pathTemplate) + '}());');
      _handlebars2.default.templates[templateId] = _handlebars2.default.template(tmpl);
    }
  }, {
    key: 'loadHbsTemplates',
    value: function loadHbsTemplates() {
      var pathTemplate = _path2.default.join(_.config.root, _.config.templates.url, 'hbs');

      try {
        var directory = _fsExtra2.default.lstatSync(pathTemplate);
        if (!directory.isDirectory()) {
          _mkdirp2.default.sync(pathTemplate);
        }
      } catch (e) {
        _mkdirp2.default.sync(pathTemplate);
      }

      _fsExtra2.default.readdirSync(pathTemplate).forEach(function (file) {
        if (file.indexOf('.hbs') > -1) {
          var originalTemplatePath = _path2.default.join(_.config.root, _.config.templates.url) + '/' + file.replace('.hbs', '.' + _.config.files.templates.extension);

          try {
            var originalTemplateStat = _fsExtra2.default.statSync(originalTemplatePath);
            var originalTemplateMdate = originalTemplateStat.mtime;
            var stat = _fsExtra2.default.statSync(_path2.default.join(pathTemplate, file));
            var mdate = stat.mtime;

            // if the original template has been updated after precompilation, I delete the precompiled file
            // else I add it to the hbs template array
            if (originalTemplateMdate > mdate) {
              _fsExtra2.default.unlinkSync(_path2.default.join(pathTemplate, file));
            } else {
              var tmpl = eval('(function(){return ' + _fsExtra2.default.readFileSync(_path2.default.join(pathTemplate, file)) + '}());');
              _handlebars2.default.templates[file.replace('.hbs', '')] = _handlebars2.default.template(tmpl);
            }
          } catch (err) {
            console.log('The original template has not been found or the hbs template is corrupted');
            console.log(originalTemplatePath);
            console.log(err);
          }
        }
      });
    }
  }], [{
    key: 'instance',
    get: function get() {
      if (!this[singleton]) {
        this[singleton] = new Manager(singletonEnforcer);
      }
      return this[singleton];
    }
  }]);

  return Manager;
}();

exports.default = Manager;