'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _xss = require('xss');

var _xss2 = _interopRequireDefault(_xss);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _package = require('../../../package');

var _package2 = _interopRequireDefault(_package);

var _cli = require('../../cli');

var _editor = require('../controllers/editor');

var _abeLocale = require('../helpers/abe-locale');

var _abeLocale2 = _interopRequireDefault(_abeLocale);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var route = function route(req, res, next) {
  if (req.query.filePath) {
    var testXSS = (0, _xss2.default)(req.query.filePath, {
      whiteList: [],
      stripIgnoreTag: true
    });
    if (testXSS !== req.query.filePath) {
      res.redirect('/abe/' + req.params[0] + '?filePath=' + testXSS);
      return;
    }
  }
  _cli.abeExtend.hooks.instance.trigger('beforeRoute', req, res, next);
  if (typeof res._header !== 'undefined' && res._header !== null) return;

  var templatePath = req.params[0];

  var filePath = null;
  if (typeof req.query.filePath !== 'undefined' && req.query.filePath !== null) {
    filePath = _path2.default.join(_cli.config.root, _cli.config.draft.url, req.query.filePath.replace(_cli.config.root));
  }
  var debugJson = req.query.debugJson && req.query.debugJson == 'true' ? true : false;
  var debugJsonKey = req.query.key ? req.query.key : false;
  var debugHtml = req.query.debugHtml && req.query.debugHtml == 'true' ? true : false;

  var isHome = true;

  var p = new Promise(function (resolve) {

    if (templatePath !== null && filePath !== null) {
      var jsonPath = null;
      var linkPath = null;
      isHome = false;

      var filePathTest = _cli.cmsData.revision.getDocumentRevision(req.query.filePath);
      if (typeof filePathTest !== 'undefined' && filePathTest !== null) {
        jsonPath = filePathTest.path;
        linkPath = filePathTest.abe_meta.link;
      }

      if (jsonPath === null || !_cli.coreUtils.file.exist(jsonPath)) {
        res.redirect('/abe/');
        return;
      }

      (0, _editor.editor)(templatePath, jsonPath, linkPath).then(function (result) {
        resolve(result);
      }).catch(function (e) {
        console.error(e);
      });
    } else {
      resolve({
        obj: {},
        manager: {}
      });
    }
  }).catch(function (e) {
    console.error(e); // "oh, no!"
  });

  p.then(function (result) {
    var obj = result;
    var manager = {};

    manager.home = {
      files: _cli.Manager.instance.getList()
    };

    manager.list = _cli.Manager.instance.getStructureAndTemplates();
    manager.editConfig = req.app.get('config');
    manager.config = JSON.stringify(_cli.config);

    var _hasBlock = obj ? obj.hasBlock : false;
    var _hasSingleBlock = obj ? obj.hasSingleBlock : false;
    var _template = filePath ? '/page/' + req.params[0] + ('?filePath=' + req.query.filePath) : false;
    var _form = obj ? obj.form : false;
    var _json = obj ? obj.json : false;
    var _text = obj ? obj.text : false;
    // var _file = (tplUrl) ? tplUrl.draft.file : false
    var _filePath = req.query.filePath ? req.query.filePath : false;
    if (_filePath) {
      _filePath = '/' + _filePath.replace(/^\/+/, '');
    }

    var pageHtml = '';
    if (typeof _json !== 'undefined' && _json !== null && typeof _json.abe_meta !== 'undefined' && _json.abe_meta !== null) {
      var text = _cli.cmsTemplates.template.getTemplate(_json.abe_meta.template);
      var page = new _cli.Page(_json.abe_meta.template, text, _json, false);
      pageHtml = page.html.replace(/"/g, '"').replace(/'/g, '\'').replace(/<!--/g, '<ABE!--').replace(/-->/g, '--ABE>');
    }

    var EditorVariables = {
      pageHtml: pageHtml,
      isHome: isHome,
      abeUrl: '/abe/',
      test: JSON.stringify(_abeLocale2.default),
      text: _abeLocale2.default,
      templatePath: req.params[0],
      template: _template,
      hasSingleBlock: _hasSingleBlock,
      hasBlock: _hasBlock,
      form: _form,
      urlToSaveFile: _filePath,
      folderToSaveFile: _filePath ? _path2.default.dirname(_filePath) : '',
      // tplName: _file,
      json: _json,
      config: _cli.config,
      Locales: _cli.coreUtils.locales.instance.i18n,
      manager: manager,
      express: {
        res: res,
        req: req
      },
      abeVersion: _package2.default.version,
      nonce: '\'nonce-' + res.locals.nonce + '\''
    };
    EditorVariables = _cli.abeExtend.hooks.instance.trigger('afterVariables', EditorVariables);

    if (debugJson) {
      var dj = _json;
      if (debugJsonKey && typeof dj[debugJsonKey] !== 'undefined' && dj[debugJsonKey] !== null) {
        dj = dj[debugJsonKey];
      }
      res.set('Content-Type', 'application/json');
      res.send(JSON.stringify(dj));
    } else if (debugHtml) {
      res.set('Content-Type', 'text/plain');
      res.send(_text);
    } else {
      res.render(_cli.config.abeEngine, EditorVariables);
    }
  }).catch(function (e) {
    console.log('error', e);
  });
};

exports.default = route;